from eurydike.eventdetectors.abstracteventdetector import AbstractEventDetector


class OnBand(AbstractEventDetector):
    _upper_threshold = None
    _lower_threshold = None

    def __init__(self, config, parent_logger, mqtt_client):
        AbstractEventDetector.__init__(self, config, parent_logger, mqtt_client)
        self._upper_threshold = self._config["upper-threshold"]
        self._lower_threshold = self._config["lower-threshold"]
        if self._lower_threshold >= self._upper_threshold:
            self._logger.error("OnBand.__init__ ({}) - lower limit ({}) must be smaller than upper limit({}).".
                               format(self._name, self._lower_threshold, self._upper_threshold))
            raise ValueError("OnBand.__init__ ({}) - lower limit ({}) must be smaller than upper limit({}).".
                               format(self._name, self._lower_threshold, self._upper_threshold))

    def _is_violation(self, value):
        return not (self._lower_threshold <= value <= self._upper_threshold)



from pelops.abstractmicroservice import AbstractMicroservice
from eurydike.eventdetectors.eventdetectorfactory import EventDetectorFactory


class EventDetectionManager(AbstractMicroservice):
    _event_detectors = None

    def __init__(self, config, mqtt_client=None, logger=None):
        AbstractMicroservice.__init__(self, config, "eventdetectors", mqtt_client=mqtt_client, logger=logger)
        self._event_detectors = EventDetectorFactory.create_elements(self._config, self._logger, self._mqtt_client)

    def _start(self):
        for event in self._event_detectors:
            event.start()

    def _stop(self):
        for event in self._event_detectors:
            event.stop()



import logging


class AbstractEventDetector:
    _config = None
    _logger = None
    _name = None
    _mqtt_client = None
    _event_active = None
    _topic_sub = None
    _topic_pub = None
    _responses = None

    def __init__(self, config, parent_logger, mqtt_client):
        self._config = config
        self._logger = parent_logger.getChild(__name__)
        self._logger.info("{}.__init__ - initializing".format(self.__class__.__name__))
        self._logger.debug("{}.__init__ - configuration: {}".format(self.__class__.__name__, self._config))

        self._name = self._config["name"]

        self._mqtt_client = mqtt_client
        self._topic_pub = self._config["topic-pub"]
        self._topic_sub = self._config["topic-sub"]
        self._responses = {}
        for k, v in self._config["responses"].items():
            if v is None or v == "":
                continue
            self._responses[k] = v

        self._event_active = False

    def _message_handler(self, value):
        try:
            value = int(value)
        except ValueError:
            value = float(value)

        if self._is_violation(value):
            # threshold is violated
            if not self._event_active:
                self._logger.info("{}._message_handler ({}) - threshold violation detected (value: {}).".
                                  format(self.__class__.__name__, self._name, value))
                try:
                    self._mqtt_client.publish(self._topic_pub, self._responses["on-violation"])
                except KeyError:
                    pass
            self._event_active = True
        else:
            if self._event_active:
                self._logger.info("{}._message_handler ({}) - threshold restored (value: {}).".
                                  format(self.__class__.__name__, self._name, value, ))
                try:
                    self._mqtt_client.publish(self._topic_pub, self._responses["on-restoration"])
                except KeyError:
                    pass
            self._event_active = False

    def _is_violation(self, value):
        raise NotImplementedError

    def start(self):
        self._mqtt_client.subscribe(self._topic_sub, self._message_handler)

    def stop(self):
        self._mqtt_client.unsubscribe(self._topic_sub, self._message_handler)



from eurydike.eventdetectors.abstracteventdetector import AbstractEventDetector
from enum import Enum


class Comparator(Enum):
    GREATERTHAN = (lambda x,y : x > y),
    LOWERTHAN = (lambda x,y: x < y),
    EQUALTO = (lambda x,y: x == y),

    @classmethod
    def get_enum(cls, text, logger):
        if text.lower()=="greaterthan" or text.lower()=="gt" or text.lower()==">":
            return cls.GREATERTHAN
        elif text.lower()=="lowerthan" or text.lower()=="lt" or text.lower()=="<":
            return cls.LOWERTHAN
        elif text.lower()=="equalto" or text.lower()=="==":
            return cls.EQUALTO
        else:
            logger.error("Comperator.get_enum - unkown value '{}'.".format(text))
            raise ValueError("Comperator.get_enum - unkown value '{}'.".format(text))

    def __call__(self, *args, **kwargs):
        return self.value[0](*args, **kwargs)

class OnThreshold(AbstractEventDetector):
    _threshold = None
    _comparator = None

    def __init__(self, config, parent_logger, mqtt_client):
        AbstractEventDetector.__init__(self, config, parent_logger, mqtt_client)
        self._comparator = Comparator.get_enum(self._config["comparator"], self._logger)
        self._threshold = self._config["threshold"]

    def _is_violation(self, value):
        return self._comparator(value, self._threshold)


